table_name = r"(?<=insert[\W]into[\W])(\w+)"
column_names = r"(\([-'`\d\w, ]+\))(?:\W+)(?:values)"
values_starting_position = (
    r"(?:insert[\W]+into[\W]+[ \w\d_]+\([,\w\d' _\n]+\)[\W]+)(?:values[ \n]+)"
)
comma_between_multiple_values = r"(?:\)[\W]*)(,)(?:[\W]*\()"

round_br_op = "("
round_br_cl = ")"

curly_br_op = "{"
curly_br_cl = "}"

single_quote = "'"
double_quote = '"'
comma = ","
semi_colon = ";"



from sqlcsvsql.sqlparsers.parsers import Parser


def run(file_path, output=None):
    if output is None:
        output = file_path + ".csv"
    with open(file_path, "r", encoding="utf8") as file:
        sql = file.read()
    p = Parser(sql, True)
    p.parse()
    p.to_csv(output)



import csv
import logging
import sys

config = {}
output = {}
logger = logging.getLogger(__name__)


def run(table_name, file_name, is_multiline=False, output_file=None, surround=None):
    """
    driver method.

    Parameters
    ----------
    table_name: str
        name of the table to use in DML
    file_name: str
        input csv file
    is_multiline: bool
        whether to generate multiple insert statements or single one.
    output_file: str
        output filepath with extension
    surround: str
        quotation flags for VALUES (...) in generated sql

    Returns
    -------
    Nothing. Saves the file on disk
    """
    if output_file is None:
        output_file = file_name + ".sql"
    config["table"] = table_name

    table = read_csv(file_name)
    config["columns"] = list(table[0].keys())
    config["is_multiline"] = is_multiline
    output[config["table"]] = {"values": [], "insert_into": ""}
    setup_quote_chars(surround)
    write_to_sql(output_file, prepare_sql(table))


def setup_quote_chars(surround):
    config["quote"] = {}
    if surround is not None:
        surrounds = surround.split(",")
        if len(surrounds) != len(config["columns"]):
            logger.error(
                "surround parameter(s) %s does not match columns %s",
                len(surrounds),
                len(config["columns"]),
            )
            sys.exit(0)

        for idx, s in enumerate(surrounds):
            config["quote"][config["columns"][idx]] = "'"
            if s == "n":
                config["quote"][config["columns"][idx]] = ""
    else:
        for col in config["columns"]:
            config["quote"][col] = "'"


def read_csv(file_name):
    """
    read csv

    Returns
    -------
    rows: List[Dict]
        list of dict. each dict contains csv in {key:value} format
    """
    rows = []
    with open(file_name) as csvfile:
        reader = csv.DictReader(csvfile)
        for row in reader:
            rows.append(row)
    return rows


def write_to_sql(file_name, content):
    with open(file_name, "w") as f:
        f.write(content)


def prepare_insert_header_statement(table, cols):
    """
    Returns first-half of INSERT DML, i.e `INSERT INTO TABLE (COLUMN...) VALUES`
    """
    cols_sql = f"({', '.join([c.strip() for c in cols])})"
    return f"INSERT INTO {table} {cols_sql} VALUES"


def format_value(value, quote):
    """
    decides the following for each values
        - null or not
        - use quotation or not
    """
    if value == "null":
        return value
    return quote + str(value) + quote


def prepare_statement(values):
    values_sql = f"""({', '
        .join([format_value(v, config['quote'][config['columns'][i]]) for i, v in enumerate(values)])})"""
    return values_sql


def prepare_values(line):
    """
    converts line in a csv to sql
    """
    table_name = config["table"]
    values = []
    for k in config["columns"]:
        values.append(line[k])
    statement = prepare_statement(values)
    output[table_name]["values"].append(statement)


def prepare_sql(table):
    """Generates the sql given a csv table"""
    sql = ""
    for row in table:
        prepare_values(row)
    insert_statement = prepare_insert_header_statement(
        config["table"], config["columns"]
    )
    if not config["is_multiline"]:
        sql = ", ".join(output[config["table"]]["values"])
        sql = insert_statement + " " + sql + ";"
        return sql

    for v in output[config["table"]]["values"]:
        sql = sql + "\n" + insert_statement + " " + v + ";"
    return sql



import re
import argparse
import logging

from sqlcsvsql.converters import sql_to_csv, csv_to_sql

REGEX_CSV = r".csv$"
REGEX_SQL = r".sql$"

parser = argparse.ArgumentParser()
parser.add_argument("-f", "--filepath", help="SQL or CSV file path")
parser.add_argument("-t", "--table-name", help="Table name for csv file")
parser.add_argument("-o", "--output", help="Output filepath. Must end with extension")
parser.add_argument(
    "-m",
    "--statement_mode",
    help="single or multiline DML to generate",
    choices=("single", "multi"),
)
parser.add_argument(
    "-s",
    "--surround",
    help="""surround values with single quote. For a CSV with columns A,B,C 
    if only B column's values should be surrounded with quotes, then use 
    ```--surround n,y,n```. By default, all values are surrounded. 
    """,
    choices=("y", "n"),
)


def main(arguments=None):
    logging.basicConfig(format="[%(levelname)s %(asctime)s %(name)s]: %(message)s")

    if arguments is None:
        parser.print_help()

    if arguments.filepath:
        if file_type(arguments.filepath) == "sql":
            sql_to_csv.run(arguments.filepath, arguments.output)
        elif file_type(arguments.filepath) == "csv":
            csv_to_sql.run(
                arguments.table_name,
                arguments.filepath,
                arguments.statement_mode == "multi",
                arguments.output,
                arguments.surround
            )


def file_type(filename):
    csv_pattern = re.compile(REGEX_CSV, re.IGNORECASE)
    sql_pattern = re.compile(REGEX_SQL, re.IGNORECASE)

    m = csv_pattern.search(filename)
    if m:
        return "csv"
    m = sql_pattern.search(filename)
    if m:
        return "sql"
    return None


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)



import re
import csv
import logging

from sqlcsvsql.sqlparsers import reg_exp

logger = logging.getLogger(__name__)


class Parser:
    output = {"table_name": "", "values": [], "columns": []}

    def __init__(self, sql, multiple_stmt=False) -> None:
        self.sql = sql
        self.multiple_stmt = multiple_stmt

    def parse(self):
        self.output["table_name"] = get_table_name(self.sql)
        self.output["column_names"] = get_column_names(self.sql)
        all_insert_statements = get_all_insert_stmt(self.sql)
        self.output["statements"] = all_insert_statements

        for statement in all_insert_statements:
            starting_positions = get_values_starting_pos(statement)
            for pos in starting_positions:
                partial_sql = statement[pos:]
                self.output["values"] += parse_multiple_value_statement(partial_sql)
        return

    def to_csv(self, filename):
        with open(filename, "w", encoding="utf-8") as csvfile:
            writer = csv.writer(csvfile, quoting=csv.QUOTE_NONNUMERIC)
            writer.writerow(self.output["column_names"])
            for values in self.output["values"]:
                # values = (v.encode("utf-8") for v in values)
                writer.writerow(values)


def get_table_name(sql: str, multiple_stmt=False):
    """
    returns table name from sql. assumes only one table is in the sql
    """
    expression = re.compile(reg_exp.table_name, re.IGNORECASE)
    names = expression.findall(sql)
    if len(names) == 0:
        logger.error("no table name found")
    return names[0]


def get_values_starting_pos(sql: str):
    """
    finds the position of values starting in a single statement.

    Example
    -------
    for sql `INSERT INTO ABC(id,name) VALUES (1,'a'),(2,'c');`
    the method will return the idx of whitespace after `VALUES` keyword.

    :param sql
    :return: list of int
    """
    positions = []
    expression = re.compile(
        reg_exp.values_starting_position, re.IGNORECASE | re.MULTILINE
    )
    iteration = expression.finditer(sql)
    if iteration is None:
        logger.error("no values here")
    for match in iteration:
        positions.append(match.end())
    return positions


def parse_multiple_value_statement(sql: str):
    """
    1. if , without last item quote in b : make tuple
    2. if ( with last item in b is quote: text
    3. if ) with last item in b is quote: text
    4. if ' with last item in b is quote: pop, text
    5. if ) with last item in b is (: tuple, pop
    6. if , without any item in b, do nothing
    """
    list_values = []

    brackets = []
    current_substr = ""
    sub_str_list = []
    for c in sql:
        if c == reg_exp.comma:
            if len(brackets) == 0:
                continue
            if last_item(brackets) != reg_exp.single_quote:
                sub_str_list.append(current_substr.strip())
                current_substr = ""
                continue
        if c == reg_exp.round_br_op:
            if last_item(brackets) != reg_exp.single_quote:
                brackets.append(reg_exp.round_br_op)
                continue
        if c == reg_exp.round_br_cl:
            if last_item(brackets) == reg_exp.round_br_op:
                brackets.pop()

                sub_str_list.append(current_substr.strip())
                list_values.append(tuple(sub_str_list))

                sub_str_list = []
                current_substr = ""
                continue
        if c == reg_exp.single_quote:
            if last_item(brackets) == reg_exp.single_quote:
                brackets.pop()
            else:
                brackets.append(reg_exp.single_quote)
            continue

        current_substr += c
    return list_values


def get_all_insert_stmt(sql: str):
    """
    :param sql: complete sql read from file
    :return: list of sql separated by `;`
    """
    brackets = []
    cursor = 0
    positions = []
    stmts = []
    for c in sql:
        cursor += 1
        if c == reg_exp.semi_colon:
            if len(brackets) == 0:
                positions.append(cursor)
                continue
        if c == reg_exp.single_quote:
            if last_item(brackets) == reg_exp.single_quote:
                brackets.pop()
            else:
                brackets.append(reg_exp.single_quote)
                continue

    positions.insert(0, 0)
    positions.append(len(sql))
    for idx in range(0, len(positions) - 1):
        stmts.append(sql[positions[idx] : positions[idx + 1]].strip())
    return stmts


def get_column_names(sql: str):
    p = re.compile(reg_exp.column_names, re.IGNORECASE)
    m = p.findall(sql)
    if len(m) == 0:
        return None
    m = m[0][1:-1]
    return [col.strip() for col in m.split(",")]


def last_item(arr):
    if len(arr) > 0:
        return arr[-1]
    return ""
